/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package studentcatalog;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author apple
 */
public class CatalogueImpl implements Catalogue {
    private Map<String,List<Results>> results = new HashMap<String,List<Results>>();

    public Set<String> getStudents() {
        return Collections.unmodifiableSet(results.keySet());
    }

    public List<Results> getGrades(String student) {
        return Collections.unmodifiableList(results.get(student));
    }

    public List<Integer> getGrades(String student, String study) {
        List<Integer> grades=new LinkedList<Integer>();
        for(Results result:results.get(student)){
            if(result.getStudy().equals(study)){
                grades.add(result.getGrade());
            }
        }
        return grades;
    }

    public void store(String filename) throws CatalogueException {
        BufferedWriter output;
        try {
            output = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename),"ISO-8859-2"));
        } catch (FileNotFoundException ex) {
            throw new CatalogueException("Nelze otevrit soubor:",ex);
        } catch(UnsupportedEncodingException ex1){
            throw new CatalogueException("Spatne kodovani.",ex1);
        }
        for(String student:results.keySet()){
            try {
                output.write(student);
                output.newLine();
            } catch (IOException ex) {
                throw new CatalogueException("Nelze zapsat data:",ex);
            }
            writeGrades(output,getGrades(student));
        }
        try {
            output.close();
        } catch (IOException ex) {
            throw new CatalogueException("Nelze zavrit soubor.",ex);
        }
    }

    public void load(String filename) throws CatalogueException{
        BufferedReader input;
        try {
            input = new BufferedReader(new InputStreamReader(new FileInputStream(filename),"ISO-8859-2"));
        } catch (FileNotFoundException ex) {
            throw new CatalogueException("Nelze otevrit vstup.",ex);
        } catch(UnsupportedEncodingException ex1){
            throw new CatalogueException("Neplatne kodovani.",ex1);
        }
        String line;
        try {
            line = input.readLine();
            while (line != null) {
               String student = line;
               List<Results> results = getResults(input);
            }
        } catch (IOException ex) {
            try {
                input.close();
            } catch (IOException ex1) {
                throw new CatalogueException("Nelze zavrit souboru po chybe.",ex1);
            }
            throw new CatalogueException("Nelze nacist data.",ex);
        }
        try {
            input.close();
        } catch (IOException ex) {
            throw new CatalogueException("Nelze zavrit soubor.",ex);
        }

    }

    public void add(String student, String study, int grade) {
        List<Results> grades;

        if(results.containsKey(student)){
            grades = results.get(student);
        }else{
            grades = new LinkedList<Results>();
        }
        grades.add(new ResultsImpl(study,grade));
        results.put(student, grades);
    }

    private List<Results> getResults(BufferedReader input) throws CatalogueException {
        List<Results> results = new LinkedList<Results>();
        String line;

        try {
            line = input.readLine();
        } catch (IOException ex) {
            throw new CatalogueException("Nelze nacist predmety",ex);
        }
        if(line != null){
            String[] courses=line.split(",");
            for(String course:courses){
                String[] strResult = course.split(":");
                if(strResult.length!=2){
                    throw new CatalogueException("Neplatny format vysledku u predmetu.");
                }
                int grade = Integer.parseInt(strResult[1]);
                results.add(new ResultsImpl(strResult[0],grade));
            }
        }else{
            throw new CatalogueException("Chybi vysledky.");
        }
        return results;
    }

    private void writeGrades(BufferedWriter output, List<Results> grades) throws CatalogueException {
        StringBuilder outputString = new StringBuilder();
        for(Iterator<Results> it = grades.iterator() ; it.hasNext() ;){
            Results result = it.next();
            outputString.append(result.getStudy()+":"+result.getGrade());
            if(it.hasNext()){
                outputString.append(",");
            }else{
                outputString.append("\n");
            }
        }
        try {
            output.write(outputString.toString());
        } catch (IOException ex) {
            throw new CatalogueException("Nelze zapsat data.",ex);
        }
    }

    @Override
    public String toString(){
        StringBuilder desc = new StringBuilder();
        for(String student:this.results.keySet()){
            desc.append(student+":\n");
            for(Results result:this.results.get(student)){
                desc.append(result.getStudy()+": "+result.getGrade()+"\t");
            }
        }
        return desc.toString();

    }

}
